<?php
/**
 * @file
 * Front end functions for the DIBS module
 */

/**
 * Page with hidden form fields that redirects to the DIBS payment
 *
 * @param array $transaction
 *
 * @return string
 */
function dibs_frontend_redirect_page($transaction) {
  $transaction['payment_price'] = (float) $transaction['payment_price'];

  // Making some checks to check if the transaction is valid.
  if (!empty($transaction['api_module']) && strlen($transaction['api_delta'])) {
    $info = dibs_invoke($transaction['api_module'], 'info', $transaction['api_delta']);
    if (empty($info) || !is_array($info)) {
      drupal_set_message(t('Error redirecting to DIBS: api_module and/or api_delta is not valid!'), 'error');
      return '';
    }
  }
  elseif (empty($transaction['api_module']) || !strlen($transaction['api_delta'])) {
    drupal_set_message(t('Error redirecting to DIBS: api_module or api_delta is missing!'), 'error');
    return '';
  }
  elseif ($transaction['payment_status'] > 0) {
    drupal_set_message(t('Error redirecting to DIBS: the payment was already made!'), 'error');
    return '';
  }

  // Loading settings.
  $settings = dibs_load_settings($transaction['api_module'], $transaction['api_delta']);

  if (empty($settings)) {
    drupal_set_message(t('DIBS module is not configured for module %module, delta %delta.', array('%module' => $transaction['api_module'], '%delta' => $transaction['api_delta'])), 'error');
    return t('Redirect could not be performed due to missing configuration settings.');
  }

  // Getting redirect form.
  $form = drupal_get_form('dibs_frontend_redirect_form', $settings, $transaction);

  // Adding autosubmit javascript.
  drupal_add_js("
    if (Drupal.jsEnabled) {
      $(document).ready($('#dibs-frontend-redirect-form').submit());
    }
  ", 'inline', 'footer');

  return theme('dibs_redirect_page', $form, $settings, $transaction);
}

/**
 * Form builder; Building the payment form and submits
 * it to DIBS with javascript
 *
 * @param array $form_state
 * @param array $settings
 * @param array $data
 *
 * @return array
 */
function dibs_frontend_redirect_form($form_state, $settings, $data) {
  if ($data['payment_retry_count']) {
    $data['payment_order_id'] .= '-' . $data['payment_retry_count'];
  }
  
  $subunit = !empty($data['params']['disable_subunit_calc']) ? 1 : 100;  

  // Creating form fields.
  // Required fields.
  $form['merchant'] = array(
    '#type' => 'hidden',
    '#value' => $settings['general']['merchant'],
  );
  $form['amount'] = array(
    '#type' => 'hidden',
    '#value' => $data['payment_price'] * $subunit,
  );
  $form['currency'] = array(
    '#type' => 'hidden',
    '#value' => $settings['general']['currency'],
  );
  $form['lang'] = array(
    '#type' => 'hidden',
    '#value' => $settings['general']['lang'],
  );
  $form['orderid'] = array(
    '#type' => 'hidden',
    '#value' => $data['payment_order_id'],
  );
  $form['uniqueoid'] = array(
    '#type' => 'hidden',
    '#value' => 1,
  );
  $form['accepturl'] = array(
    '#type' => 'hidden',
    '#value' => url('payment/dibs/accept/' . $data['api_hash'], array('absolute' => TRUE)),
  );
  $form['cancelurl'] = array(
    '#type' => 'hidden',
    '#value' => url('payment/dibs/cancel/' . $data['api_hash'], array('absolute' => TRUE)),
  );
  $form['callbackurl'] = array(
    '#type' => 'hidden',
    '#value' => url('payment/dibs/callbackok/' . $data['api_hash'], array('absolute' => TRUE)),
  );
  $form['api_module'] = array(
    '#type' => 'hidden',
    '#value' => $data['api_module'],
  );
  $form['api_delta'] = array(
    '#type' => 'hidden',
    '#value' => $data['api_delta'],
  );

  // Optional fields.
  if (!empty($settings['general']['test_mode'])) {
    $form['test'] = array(
      '#type' => 'hidden',
      '#value' => 1,
    );
  }
  if (!empty($settings['general']['account'])) {
    $form['account'] = array(
      '#type' => 'hidden',
      '#value' => $settings['general']['account'],
    );
  }
  if (!empty($settings['advanced']['capturenow'])) {
    $form['capturenow'] = array(
      '#type' => 'hidden',
      '#value' => 1,
    );
  }
  if (!empty($settings['advanced']['calcfee'])) {
    $form['calcfee'] = array(
      '#type' => 'hidden',
      '#value' => 1,
    );
  }
  
  // Order and payment info fields
  // If order_info_long contains data are we not sending order_info_short to DIBS.
  // The reason for this is a bug in the DIBS administration. If both are sent,
  // will only the short version show up in the administration.
  if (!empty($data['order_info_long']) && is_array($data['order_info_long'])) {
    $order_lines = dibs_create_order_line_fields($data['order_info_long']);
    $form = array_merge($form, $order_lines);
  }
  elseif (!empty($data['order_info_short'])) {
    $form['ordertext'] = array(
      '#type' => 'hidden',
      '#value' => $data['order_info_short'],
    );
  }
  if (!empty($data['order_price_info']) && is_array($data['order_price_info'])) {
    $price_lines = dibs_create_price_line_fields($data['order_price_info']);
    $form = array_merge($form, $price_lines);
  }

  // Delivery line fields.
  $delivery_lines = dibs_create_delivery_line_fields($data);
  $form = array_merge($form, $delivery_lines);  

  // Payment Window fields.
  if ($settings['general']['type'] == 'pay') {
    // Changing parameters with new names.
    $form['orderId'] = $form['orderid'];
    $form['acceptReturnUrl'] = $form['accepturl'];
    $form['cancelReturnUrl'] = $form['cancelurl'];
    $form['callbackUrl'] = $form['callbackurl'];
    $form['addFee'] = $form['calcfee'];
    $form['language'] = $form['lang'];
    unset($form['orderid'], $form['accepturl'], $form['cancelurl'], $form['callbackurl'], $form['calcfee'], $form['lang']);
    
    if (!empty($settings['paymentwindow']['hmac'])) {
      $form['MAC'] = array(
        '#type' => 'hidden',
        '#value' => dibs_generate_mac($form, $settings['paymentwindow']['hmac_key']),
      );
    }    
    
    $form['#action'] = 'https://sat1.dibspayment.com/dibspaymentwindow/entrypoint';
  }
  // Flex Window fields.
  elseif ($settings['general']['type'] == 'flex') {
    if (!empty($settings['flexwindow']['md5'])) {
      $md5_key = md5($settings['flexwindow']['md5_key2'] . md5($settings['flexwindow']['md5_key1'] . 'merchant=' . $settings['general']['merchant'] . '&orderid=' . $data['payment_order_id'] . '&currency=' . $settings['general']['currency'] . '&amount=' . $data['payment_price'] * 100));
      $form['md5key'] = array(
        '#type' => 'hidden',
        '#value' => $md5_key,
      );
    }
    if (!empty($settings['flexwindow']['color'])) {
      $form['color'] = array(
        '#type' => 'hidden',
        '#value' => $settings['flexwindow']['color'],
      );
    }
    if (!empty($settings['flexwindow']['decorator']) && ($settings['flexwindow']['decorator'] != 'custom')) {
      $form['decorator'] = array(
        '#type' => 'hidden',
        '#value' => $settings['flexwindow']['decorator'],
      );
    }
    else {
      $form['decorator'] = array(
        '#type' => 'hidden',
        '#value' => '',
      );
    }
    if (!empty($settings['flexwindow']['voucher'])) {
      $form['voucher'] = array(
        '#type' => 'hidden',
        '#value' => $settings['flexwindow']['voucher'] ? 'yes' : '',
      );
    }
    $form['#action'] = 'https://payment.architrade.com/paymentweb/start.action';
  }
  
  // Additional form fields.
  $form['submit'] = array('#type' => 'submit', '#value' => t('Continue to DIBS payment'));

  // Executing the hook_dibsapi_form_alter
  $func = $data['api_module'] . '_dibsapi_form_alter';
  if (function_exists($func)) {
    $func($form, $data['api_delta']);
  }

  return $form;
}

/**
 * The cancel page where the user has the possibility to go back to the
 * payment again
 *
 * @param array $transaction
 *
 * @return string
 */
function dibs_frontend_cancel_page($transaction) {
  $transaction['payment_price'] = (float) $transaction['payment_price'];

  // Making some checks to check if the transaction is valid.
  if (!empty($transaction['api_module']) && strlen($transaction['api_delta'])) {
    $info = dibs_invoke($transaction['api_module'], 'info', $transaction['api_delta']);
    if (empty($info) || !is_array($info)) {
      drupal_set_message(t('Error redirecting to DIBS: api_module and/or api_delta is not valid!'), 'error');
      return '';
    }
  }
  elseif (empty($transaction['api_module']) || !strlen($transaction['api_delta'])) {
    drupal_set_message(t('Error redirecting to DIBS: api_module or api_delta is missing!'), 'error');
    return '';
  }
  elseif ($transaction['payment_status'] > 0) {
    drupal_set_message(t('Error redirecting to DIBS: the payment has already made!'), 'error');
    return '';
  }

  // Invoking the module that implemented the payment.
  dibs_invoke($transaction['api_module'], 'transaction_cancel', $transaction['api_delta'], $transaction);

  // Loading settings.
  $settings = dibs_load_settings($transaction['api_module'], $transaction['api_delta']);

  // Getting redirect form.
  $form = drupal_get_form('dibs_frontend_cancel_form', $settings, $transaction);

  return theme('dibs_cancel_page', $form, $settings, $transaction);
}

/**
 * Form builder; Building the cancel form
 *
 * @param array $form_state
 * @param array $settings
 * @param array $data
 *
 * @return array
 *
 * @ingroup forms
 * @see dibs_frontend_cancel_form_submit()
 */
function dibs_frontend_cancel_form($form_state, $settings, $data) {
  $form['hash'] = array('#type' => 'value', '#value' => $data['api_hash']);
  $form['submit'] = array('#type' => 'submit', '#value' => t('Return to DIBS payment'));
  return $form;
}

/**
 * Process the cancel form
 *
 * @param array $form
 * @param array $form_state
 *
 * @ingroup forms
 * @see dibs_frontend_cancel_form()
 */
function dibs_frontend_cancel_form_submit($form, &$form_state) {
  $hash = $form_state['values']['hash'];

  // Loading the transaction.
  $data = dibs_load_transaction($hash, 'hash');
  $settings = dibs_load_settings($data['api_module'], $data['api_delta'], TRUE);

  // Make changes to order id if dictated by retry policy.
  $retry = isset($settings['general']['retry_handling']) ? $settings['general']['retry_handling'] : 'new_order_id';
  switch ($retry) {
    case 'new_order_id':
      // Generating order id.
      $order_id = dibs_get_next_order_id($data['api_module'], $data['api_delta']);
      // Updating transaction.
      db_query("UPDATE {dibs_transactions} SET payment_order_id = '%s' WHERE api_hash = '%s'", $order_id, $hash);
      break;

    case 'add_retry_suffix':
      $suffixed = array(
        'api_hash' => $hash,
        'payment_retry_count' => $data['payment_retry_count'] + 1,
      );
      drupal_write_record('dibs_transactions', $suffixed, array('api_hash'));
  }

  // Redirecting to the payment redirect page.
  drupal_goto('payment/dibs/redirect/' . $hash);

  exit;
}

/**
 * DIBS payment ok callback
 *
 * @param array $transaction
 */
function dibs_frontend_dibs_callback_ok($transaction) {
  if (!empty($transaction)) {
    $settings = dibs_load_settings($transaction['api_module'], $transaction['api_delta']);  

    // Payment Window callback
    if ($settings['general']['type'] == 'pay') {
      if (isset($_REQUEST['transaction'])) {
        $transact = $_REQUEST['transaction'];
        $mac_dibs = isset($_REQUEST['MAC']) ? $_REQUEST['MAC'] : '';
        $paytype = isset($_REQUEST['cardTypeName']) ? $_REQUEST['cardTypeName'] : '';      
      
        // Generating array with form values and calculate the MAC string.
        $mac_local = dibs_generate_mac_callback($_POST, $settings['paymentwindow']['hmac_key']);

        if ($mac_dibs == $mac_local || empty($settings['paymentwindow']['hmac'])) {
          // Updating transaction record with payment transaction id.
          $updated = db_query("UPDATE {dibs_transactions} SET payment_transaction_id = %d, payment_status = 1, payment_type = '%s' WHERE tid = %d", $transact, $paytype, $transaction['tid']);

          if ($updated) {
            watchdog('dibs', 'Transaction ID #@tid validated as OK.', array(
              '@tid' => $transaction['tid'],
            ), WATCHDOG_INFO);
          }

          // Reload the transaction.
          $transaction = dibs_transaction_load($transaction['tid']);

          // Invoking the module that implemented the payment.
          dibs_invoke($transaction['api_module'], 'transaction_callback', $transaction['api_delta'], $transaction);
        }
      
      }
      else {
        watchdog('dibs', 'Transaction ID not set in request data for OK callback, transaction #@tid.', array(
          '@tid' => $transaction['tid'],
        ), WATCHDOG_ERROR);
      }
    }
    // Flex Window callback
    else if ($settings['general']['type'] == 'flex') {
      if (isset($_REQUEST['transact'])) {
        $transact = $_REQUEST['transact'];
        $authkey = isset($_REQUEST['authkey']) ? $_REQUEST['authkey'] : '';
        $paytype = isset($_REQUEST['paytype']) ? $_REQUEST['paytype'] : '';
        $md5_key = md5($settings['flexwindow']['md5_key2'] . md5($settings['flexwindow']['md5_key1'] . 'transact=' . $transact . '&amount=' . ($transaction['payment_price'] * 100) . '&currency=' . $settings['general']['currency']));
        if ($md5_key == $authkey || empty($settings['flexwindow']['md5'])) {
          // Updating transaction record with payment transaction id.
          $updated = db_query("UPDATE {dibs_transactions} SET payment_transaction_id = %d, payment_status = 1, payment_type = '%s' WHERE tid = %d", $transact, $paytype, $transaction['tid']);

          if ($updated) {
            watchdog('dibs', 'Transaction ID #@tid validated as OK.', array(
              '@tid' => $transaction['tid'],
            ), WATCHDOG_INFO);
          }

          // Reload the transaction.
          $transaction = dibs_transaction_load($transaction['tid']);

          // Invoking the module that implemented the payment.
          dibs_invoke($transaction['api_module'], 'transaction_callback', $transaction['api_delta'], $transaction);
        }
      }
      else {
        watchdog('dibs', 'Transaction ID not set in request data for OK callback, transaction #@tid.', array(
          '@tid' => $transaction['tid'],
        ), WATCHDOG_ERROR);
      }
    }
  }
}

/**
 * DIBS accept page
 *
 * @param array $transaction
 */
function dibs_frontend_accept_page($transaction) {
  // Returning an error if the transaction is empty.
  if (empty($transaction)) {
    drupal_set_message(t('An error occured. The transaction key is not valid!'), 'error');
    return '';
  }

  // Unsetting the temporary session.
  unset($_SESSION['dibs']);

  // Invoking the module that implemented the payment.
  dibs_invoke($transaction['api_module'], 'transaction_accept', $transaction['api_delta'], $transaction);

  // Loading settings.
  $settings = dibs_load_settings($transaction['api_module'], $transaction['api_delta']);

  // Returning the themed output.
  return theme('dibs_accept_page', $settings, $transaction);
}
